@@ -1,5 +1,5 @@
 from __future__ import annotations
-import typing
+from typing import Dict, Tuple, Optional, Any
 import boto3
 import botocore.exceptions
 import boto3.dynamodb.conditions
@@ -44,7 +44,7 @@ class DB:
         except botocore.exceptions.ClientError as e:
             raise
 
-    def put(self, key: ItemKey, item: typing.Dict, expect_if_item_exists: typing.Dict = None):
+    def put(self, key: ItemKey, item: Dict, expect_if_item_exists: Dict = None):
         assert key.name in item
         assert not key.secondary or key.secondary.name in item
         expression_values = {}
@@ -63,7 +63,7 @@ class DB:
                 raise ExpectationNotMet() from e
             raise InternalError() from e
 
-    def update(self, key: ItemKey, item: typing.Dict, old_item: typing.Dict):
+    def update(self, key: ItemKey, item: Dict, old_item: Dict):
         assert key.name in item
         assert not key.secondary or key.secondary.name in item
 
@@ -112,6 +112,74 @@ class DB:
             raise InternalError() from e
 
 
+class DBMock:
+
+    def __init__(self, *args, **kwargs):
+        pass
+
+    _table: Dict[Tuple[str, Optional[str]], Any]
+
+    count_query_operations: int
+
+    def query_items(self, key: ItemKey):
+        DBMock.count_query_operations += 1
+        items = []
+        for found_key, found_item in DBMock._table.items():
+            if found_key[0] == key.value:
+                items.append(found_item)
+        return items
+
+    count_get_operations: int
+
+    def get_item(self, key: ItemKey, count=True):
+        if count:
+            DBMock.count_get_operations += 1
+        return DBMock._table.get(self._key_for_table(key))
+
+    count_put_operations: int
+
+    def put(self, key: ItemKey, item: Dict, expect_if_item_exists: Dict = None, count=True):
+        if expect_if_item_exists is None:
+            expect_if_item_exists = {}
+        if count:
+            DBMock.count_put_operations += 1
+        assert key.name in item
+        assert not key.secondary or key.secondary.name in item
+        existing = self.get_item(key, count=False)
+        if existing:
+            for name, value in expect_if_item_exists.items():
+                if existing.get(name) != value:
+                    raise ExpectationNotMet()
+        DBMock._table[self._key_for_table(key)] = item
+
+    count_update_operations: int
+
+    def update(self, key: ItemKey, item: Dict, old_item: Dict):
+        DBMock.count_update_operations += 1
+        self.put(key, item, count=False)
+
+    count_delete_operations: int
+
+    def delete(self, key: ItemKey):
+        DBMock.count_delete_operations += 1
+        if self._key_for_table(key) in self._table:
+            del DBMock._table[self._key_for_table(key)]
+
+    @staticmethod
+    def test_operations_count(query=0, get=0, put=0, update=0, delete=0):
+        assert DBMock.count_query_operations == query
+        assert DBMock.count_get_operations == get
+        assert DBMock.count_put_operations == put
+        assert DBMock.count_update_operations == update
+        assert DBMock.count_delete_operations == delete
+
+    @staticmethod
+    def _key_for_table(key: ItemKey):
+        if key.secondary:
+            return key.value, key.secondary.value
+        return key.value, None
+
+
 class ExpectationNotMet(ValueError):
     pass
 
